#ifndef IIT_ROBCOGEN_TEST_FEXT_H
#define IIT_ROBCOGEN_TEST_FEXT_H

#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include <iit/rbd/rbd.h>


namespace iit {
namespace robcogen {
namespace test {

/**
 * Read a list of external forces from a text file.
 *
 * Each force is a spatial force, therefore it has six coordinates.
 * The file format is one line for each force vector, each line being a sequence
 * of six numbers, separated by a blank. The file must have as many lines as the
 * number of links of the robot.
 *
 * \param source the input stream to read from
 * \param dest   the container of the external forces that will be filled
 *
 * \tparam Traits the robot traits as generated by RobCoGen
 */
template<class Traits>
void readExtForces(
        std::istream& source,
        typename Traits::FwdDynEngine::ExtForces& dest) // todo use a more generic, do not refer to forward dynamics, although it's the same thing
{
    std::string line;
    std::istringstream in;
    typename Traits::LinkID link;

    for(int i=0; i<Traits::links_count; i++) {
        std::getline(source, line);
        in.str(line);
        in.seekg(std::ios_base::beg);
        for(int f=0; f<6; f++) {
            link = (Traits::orderedLinkIDs())[i];
            in >> (dest[link](f));
        }
    }
}

template<class Traits>
void readExtForces(
        const std::string& extForcesFile,
        typename Traits::FwdDynEngine::ExtForces& fext)
{
    std::ifstream source;
    source.open(extForcesFile.c_str(), std::ios::in);
    if( !source.is_open() ) {
        throw std::runtime_error("Could not open file " + extForcesFile);
    }
    robcogen::test::readExtForces<Traits>(source, fext);
    source.close();
}

}
}
}

#endif
