
%% Test of the C++ forward dynamics generated by RobCoGen, for fixed base
%% robots.
%%
%% Usage: [ME ROY] = testcpp_fd(ROY_MODEL, EXE)
%%
%% ROY_MODEL is the robot model (a data structure) in Roy's format.
%%
%% EXE is the full path of the program running the C++ RobCoGen forward
%% dynamics for the same robot. The program must take the following command
%% line argument
%%   - status, velocity and force vectors in joint space
%%   - (optional) the full path with a text file with external forces
%%
%% That is, the program must be executable with something like the following:
%%    "<exe> <q'> <qd'> <tau'> <path to fext>"
%%
%% See readme.md for more information and general documentation on the inputs
%% of this function.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = testcpp_fd(roy_model, exe)

dof = length(roy_model.jtype);

q   = rand(dof,1);
qd  = rand(dof,1);
tau = rand(dof,1);

% q   = zeros(dof,1);
% qd  = zeros(dof,1);
% tau = zeros(dof,1);

%
% Random external forces
%
for i=1 : roy_model.NB
  fext{i} = rand(6, 1);
  %fext{i} = zeros(6, 1);
end

%
% Writes a file with the external forces
%
%%% ! WARNING ! %%% Check the readme for a note about external forces

fid = fopen('fext.txt', 'w');

% Write zeros in the first line to account for the fixed base
fprintf(fid, "0.0 0.0 0.0 0.0 0.0 0.0\n");

for i=1 : roy_model.NB
  fprintf(fid, "%f %f %f %f %f %f\n", fext{i});
end
fclose(fid);
fextFile = [pwd '/fext.txt']; % full absolute path of the external forces file

%
% Run Featherstone's routine. It wants the fext in global coordinates (sigh..),
% see the readme
%
[foo me] = system( [exe ' ' num2str([q', qd', tau']) ' ' fextFile] );
me  = str2num(me);
roy = FDab(roy_model, q, qd, tau, fext);
err = roy - me;

err( abs(err) < 1E-5 ) = 0;

disp([ roy, me, err]);



