
%% Test of the C++ forward dynamics generated by RobCoGen, for floating base
%% robots.
%%
%% Usage: [ME ROY] = testcpp_fd_fb(MODEL, EXE)
%%
%% MODEL is the robot model (a data structure) in Roy's format.
%%
%% EXE is the full path of the program running the C++ RobCoGen forward
%% dynamics for the same robot. The program must take the following command
%% line argument
%%   - status, velocity and force for each joint
%%   - coordinates of the spatial velocity of the base
%%   - (optional) the full path with a text file with external forces
%%
%% That is, the program must be executable with something like the following:
%%    "<exe> <q'> <qd'> <tau'> <base_v'> <path to fext>"
%%
%% See readme.md for more information and general documentation on the inputs
%% of this function.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = testcpp_fd_fb(roy_model, exe)

dof = length(roy_model.jtype) - 6;

q   = rand(dof,1);
qd  = rand(dof,1);
tau = rand(dof,1);

base_v = rand(6,1);

%q   = zeros(dof,1);
%qd  = zeros(dof,1);
%tau = zeros(dof,1);
%base_v = zeros(6,1);

Xfb = zeros(13,1);
Xfb(1) = 1;
Xfb(8:end,1) = base_v;

%
% Random external force vectors.
% (5 bodies are dummy links for the floating base virtual joint)
%
fext = cell(1, roy_model.NB);
for i=1 : length(fext)
  fext{i} = rand(6, 1);
  %fext{i} = zeros(6, 1);
end

% Writes a file with the external forces, to be read by the C++ executable.
% Ignore the first 5 elements, as they refer to the dummy links of the floating base.
%
fid = fopen('fext.txt', 'w');
for i=6 : length(fext)
  fprintf(fid, "%f %f %f %f %f %f\n", fext{i});
end
fclose(fid);
fextFile = [pwd '/fext.txt']; % full absolute path of the external forces file

%%% ! WARNING ! %%%
%
% RobCoGen code requires the external forces to be supplied in body coordinates.
% Roy's code unfortunately expects forces in world coordinates (and it converts
% them to body coordinates internally!).
% I haven't written a conversion as it's particularly boring.
% Therefore the test as it is now will never succeed, unless you modify slightly
% Roy's implementation of forward dynamics to expect external forces already in
% body coordinates.
%
%%% ! WARNING ! %%%

%
% Run the C++ executable, passing the various arguments
%
[foo output] = system( [exe ' ' num2str([q', qd', tau', base_v']) ' ' fextFile ] );

output    = str2num(output);
me.base_a = output(1:6);
me.qdd    = output(7:end);


%
% Executes Featherstone's forward dynamics in Matlab
%
[roy.xdfb, roy.qdd] = FDfb(roy_model, Xfb, q, qd, tau, fext);

roy.xdfb( abs(roy.xdfb) < 1E-3 ) = 0;
roy.qdd( abs(roy.qdd) < 1E-3 ) = 0;
roy.base_a = roy.xdfb(8:end);




err.base_a = roy.base_a - me.base_a;
err.qdd    = roy.qdd    - me.qdd;
err.base_a( abs(err.base_a) < 1E-5 ) = 0;
err.qdd   ( abs(err.qdd) < 1E-5 ) = 0;

disp([roy.base_a, me.base_a, err.base_a]);
disp('');
disp([roy.qdd   , me.qdd   , err.qdd   ]);


