
%% Test of the Octave inverse dynamics generated by RobCoGen, for fixed base
%% robots.
%%
%% Usage: [ME ROY] = test_id(ROY_MODEL, ROBCOGEN_MODEL)
%%
%% ROY_MODEL is the robot model (a data structure) in Roy's format.
%%
%% ROBCOGEN_MODEL is a structure with two fields, 'xm' and 'ip', respectively
%% the coordinate transforms for motion vectors and the inertia properties of
%% the links of the robot.
%%
%% See readme.md for more information and general documentation about the test
%% functions.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = test_id(roy_model, robcogen_model)

dof = length(roy_model.jtype);

q   = rand(dof,1);
qd  = rand(dof,1);
qdd = rand(dof,1);

% RobCoGen implementation:
[params consts] = paramsAndConsts(robcogen_model);

robcogen_model.xm = updateMotionTransforms(robcogen_model.xm, q, params, consts);
me.tau = inverseDynamics(robcogen_model.ip, robcogen_model.xm, qd, qdd);

% Roy's implementation
roy.tau = ID(roy_model, q, qd, qdd);

% Display the results and the differences
err = roy.tau - me.tau;
err( abs(err) < 1E-5 ) = 0;

display([roy.tau, me.tau, err]);


