
%% Test of the Octave function generated by RobCoGen to compute the Joint Space
%% Inertia Matrix (JSIM), for floating base robots.
%%
%% Usage: [ME ROY] = test_jsim_fb(ROY_MODEL, RCG)
%%
%% ROY_MODEL is the robot model (a data structure) in Roy's format.
%%
%% RCG is a structure with two fields, 'xf' and 'ip', respectively
%% the coordinate transforms for force vectors and the inertia properties of
%% the links of the robot.
%%
%% See readme.md for more information and general documentation about the test
%% functions.
%%
%% Author: Marco Frigerio
%
%%% This file is part of the Octave test functions for RobCoGen.
%%% Copyright © 2016 2017, Marco Frigerio (`marco.frigerio@iit.it`)
%%% See the LICENSE file for more information.

function [me roy] = test_jsim_fb(roy_model, rcg)

dof = length(roy_model.jtype);

q   = rand(dof,1) * 2;
qd  = rand(dof,1) * 2;

% RobCoGen implementation:
[params consts] = paramsAndConsts(rcg);

rcg.xf = updateForceTransforms (rcg.xf, q(7:end), params, consts);
% me.xm = updateMotionTransforms(me.xm, q(7:end), params, consts);

[me.M me.Ic me.F] = updateJSIM(rcg.ip, rcg.xf);
me.trunk_a = zeros(6,1);
% [me.tau me.trunk_a] = inverseDynamics(me.ip, me.xm, zeros(6,1), get_gravity(roy_model), qd, zeros(dof,1), q);

% Roy's routine
[roy.H, roy.C] = HandC( roy_model, q, qd );

% Compute the Jacobian that maps the velocity of the virtual floating base joints
%  to the spatial velocity of the robot base
[foo_qd foo_qdd J] = bodyToFBJointMotion(q(1:6), zeros(6,1), me.trunk_a);
Jinv = inv(J); % Jacobian inverse

% The Jacobian is required to convert the inertia matrix computed with Roy's
% algorithm, which works on a model of the robot where the floating base is
% emulated with 6 more joints connecting the fixed world with the base.
% The conversion is required because the acceleration (or velocity, or anything
% else) at the 6 joints it is NOT the same, in general, as the spatial velocity
% of the robot base.
% Note that this point is totally similar to the conversion joints-space/
% operational-space, for a 6-dof manipulator

roy.Ic = Jinv' * roy.H(1:6,1:6) * Jinv;  % the composite inertia of the robot base
roy.M  = roy.H(7:end, 7:end);            % the actuated-joint-space inertia matrix
roy.F  = Jinv' * roy.H(1:6, 7:end);      % the F block, upper right block of the JSIM


err.Ic = roy.Ic - me.Ic;
err.M  = roy.M  - me.M;
err.F  = roy.F  - me.F;

err.Ic( abs(err.Ic) < 1E-5 ) = 0;
err.M ( abs(err.M)  < 1E-5 ) = 0;
err.F ( abs(err.F)  < 1E-5 ) = 0;

disp(err.Ic);
disp('');
disp(err.M);
disp('');
disp(err.F);

% disp(roy.C(7:end) - me.tau)

