#ifndef IIT_ROBOT_FANCY_INERTIA_PROPERTIES_H_
#define IIT_ROBOT_FANCY_INERTIA_PROPERTIES_H_

#include <iit/rbd/rbd.h>
#include <iit/rbd/InertiaMatrix.h>
#include <iit/rbd/utils.h>

#include "declarations.h"
#include "model_constants.h"
#include "dynamics_parameters.h"

namespace iit {
namespace Fancy {
/**
 * This namespace encloses classes and functions related to the Dynamics
 * of the robot Fancy.
 */
namespace dyn {

typedef iit::rbd::InertiaMatrixDense InertiaMatrix;

class InertiaProperties {
    public:
        InertiaProperties();
        ~InertiaProperties();
        const InertiaMatrix& getTensor_link1() const;
        const InertiaMatrix& getTensor_link2() const;
        const InertiaMatrix& getTensor_link3() const;
        const InertiaMatrix& getTensor_link4() const;
        const InertiaMatrix& getTensor_link5() const;
        double getMass_link1() const;
        double getMass_link2() const;
        double getMass_link3() const;
        double getMass_link4() const;
        double getMass_link5() const;
        const iit::rbd::Vector3d& getCOM_link1() const;
        const iit::rbd::Vector3d& getCOM_link2() const;
        const iit::rbd::Vector3d& getCOM_link3() const;
        const iit::rbd::Vector3d& getCOM_link4() const;
        const iit::rbd::Vector3d& getCOM_link5() const;
        double getTotalMass() const;


        /*!
         * Fresh values for the runtime parameters of the robot Fancy,
         * causing the update of the inertia properties modeled by this
         * instance.
         */
        void updateParameters(const RuntimeInertiaParams&);

    private:
        RuntimeInertiaParams params;

        InertiaMatrix tensor_link1;
        InertiaMatrix tensor_link2;
        InertiaMatrix tensor_link3;
        InertiaMatrix tensor_link4;
        InertiaMatrix tensor_link5;
        iit::rbd::Vector3d com_link1;
        iit::rbd::Vector3d com_link2;
        iit::rbd::Vector3d com_link3;
        iit::rbd::Vector3d com_link4;
        iit::rbd::Vector3d com_link5;
};


inline InertiaProperties::~InertiaProperties() {}

inline const InertiaMatrix& InertiaProperties::getTensor_link1() const {
    return this->tensor_link1;
}
inline const InertiaMatrix& InertiaProperties::getTensor_link2() const {
    return this->tensor_link2;
}
inline const InertiaMatrix& InertiaProperties::getTensor_link3() const {
    return this->tensor_link3;
}
inline const InertiaMatrix& InertiaProperties::getTensor_link4() const {
    return this->tensor_link4;
}
inline const InertiaMatrix& InertiaProperties::getTensor_link5() const {
    return this->tensor_link5;
}
inline double InertiaProperties::getMass_link1() const {
    return this->tensor_link1.getMass();
}
inline double InertiaProperties::getMass_link2() const {
    return this->tensor_link2.getMass();
}
inline double InertiaProperties::getMass_link3() const {
    return this->tensor_link3.getMass();
}
inline double InertiaProperties::getMass_link4() const {
    return this->tensor_link4.getMass();
}
inline double InertiaProperties::getMass_link5() const {
    return this->tensor_link5.getMass();
}
inline const iit::rbd::Vector3d& InertiaProperties::getCOM_link1() const {
    return this->com_link1;
}
inline const iit::rbd::Vector3d& InertiaProperties::getCOM_link2() const {
    return this->com_link2;
}
inline const iit::rbd::Vector3d& InertiaProperties::getCOM_link3() const {
    return this->com_link3;
}
inline const iit::rbd::Vector3d& InertiaProperties::getCOM_link4() const {
    return this->com_link4;
}
inline const iit::rbd::Vector3d& InertiaProperties::getCOM_link5() const {
    return this->com_link5;
}

inline double InertiaProperties::getTotalMass() const {
    return m_link1 + m_link2 + m_link3 + m_link4 + m_link5;
}

}
}
}

#endif
